#pragma once

#include <ArmarXCore/core/ComponentPlugin.h>

#include <RobotAPI/interface/armem/mns/MemoryNameSystemInterface.h>
#include <RobotAPI/interface/armem/client/MemoryListenerInterface.h>
#include <RobotAPI/libraries/armem/core/MemoryID.h>
#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>


namespace armarx::armem::mns
{
    class MemoryNameSystemComponentPluginUser;
}


namespace armarx::armem::client::plugins
{
    /**
     * @brief A base plugin offering optional access and dependency
     * to the Memory Name System (MNS).
     */
    class MemoryNameSystemComponentPlugin : public ComponentPlugin
    {
    public:

        using ComponentPlugin::ComponentPlugin;
        virtual ~MemoryNameSystemComponentPlugin() override;

        void postCreatePropertyDefinitions(PropertyDefinitionsPtr& properties) override;

        void preOnInitComponent() override;
        void preOnConnectComponent() override;

        /**
         * @brief Indicate whether the Memory Name System (MNS) is enabled.
         */
        bool isMemoryNameSystemEnabled();
        /**
         * @brief Get the name of the MNS component.
         */
        std::string getMemoryNameSystemName();

        /**
         * @brief Get the MNS proxy.
         * @return The MNS proxy when MNS is enabled, nullptr when MNS is disabled.
         */
        mns::MemoryNameSystemInterfacePrx getMemoryNameSystem();

    public:
        static constexpr const char* PROPERTY_MNS_ENABLED_NAME = "mns.MemoryNameSystemEnabled";
        static constexpr const char* PROPERTY_MNS_NAME_NAME = "mns.MemoryNameSystemName";
    };

}


#include <ArmarXCore/core/ManagedIceObject.h>

namespace armarx::armem::client
{

    class MemoryNameSystemComponentPluginUser :
        virtual public ManagedIceObject,
        virtual public MemoryListenerInterface
    {
    protected:

        MemoryNameSystemComponentPluginUser();
        virtual ~MemoryNameSystemComponentPluginUser() override;


        virtual void memoryUpdated(const std::vector<data::MemoryID>& updatedSnapshotIDs, const Ice::Current& current) override;


        [[deprecated("Use memoryNameSystem member instead of function inherited by MemoryNameSystemComponentPluginUser.")]]
        armem::data::WaitForMemoryResult waitForMemoryServer(const std::string& memoryName);
        [[deprecated("Use memoryNameSystem member instead of function inherited by MemoryNameSystemComponentPluginUser.")]]
        armem::data::ResolveMemoryNameResult resolveMemoryServer(const std::string& memoryName);

        [[deprecated("Use memoryNameSystem member instead of function inherited by MemoryNameSystemComponentPluginUser.")]]
        armem::data::WaitForMemoryResult useMemoryServer(const MemoryID& id);
        [[deprecated("Use memoryNameSystem member instead of function inherited by MemoryNameSystemComponentPluginUser.")]]
        virtual armem::data::WaitForMemoryResult useMemoryServer(const std::string& memoryName);


    public:

        /// Only valid when enabled.
        MemoryNameSystem memoryNameSystem;

        bool memoryNameSystemEnabled = true;
        std::string memoryNameSystemName = "MemoryNameSystem";


    private:

        plugins::MemoryNameSystemComponentPlugin* plugin = nullptr;

    };
}
