#include "ConnectionManager.h"

namespace armarx::armem::server::ltm::mongodb
{
    std::mutex ConnectionManager::initializationMutex;
    bool ConnectionManager::initialized = false;
    std::map<std::string, std::unique_ptr<mongocxx::pool>> ConnectionManager::Connections = {};


    void ConnectionManager::initialize_if()
    {
        std::lock_guard l(initializationMutex); // all others have to wait until the initialization is complete
        if (!initialized)
        {
            initialized = true;
            mongocxx::instance instance{}; // This should be done only once.
        }
    }

    mongocxx::pool& ConnectionManager::Connect(const MongoDBSettings& settings)
    {
        initialize_if();

        const auto uri_str = settings.uri();
        auto it = Connections.find(uri_str);
        if (it == Connections.end())
        {
            mongocxx::uri uri(uri_str);
            auto pool = std::make_unique<mongocxx::pool>(uri);
            auto con = Connections.emplace(settings.key(), std::move(pool));
            return *con.first->second;
        }
        else
        {
            // A connection already exists. We do not need to open another one.
            return *it->second;
        }
    }

    bool ConnectionManager::ConnectionIsValid(const MongoDBSettings& settings, bool forceNewConnection)
    {
        initialize_if();

        try
        {
            if (!forceNewConnection)
            {
                auto it = Connections.find(settings.key());
                if (it != Connections.end())
                {
                    auto client = it->second->acquire();
                    auto admin = client->database("admin");
                    auto result = admin.run_command(bsoncxx::builder::basic::make_document(bsoncxx::builder::basic::kvp("isMaster", 1)));
                    return true;
                }
            }

            mongocxx::uri uri(settings.uri());
            auto client = mongocxx::client(uri);
            auto admin = client["admin"];
            auto result = admin.run_command(bsoncxx::builder::basic::make_document(bsoncxx::builder::basic::kvp("isMaster", 1)));
            return true;
        }
        catch (const std::exception& xcp)
        {
            return false;
        }
    }

    bool ConnectionManager::ConnectionExists(const MongoDBSettings& settings)
    {
        initialize_if();

        auto it = Connections.find(settings.key());
        return it != Connections.end();
    }
}
