#pragma once

#include <optional>

#include <QWidget>
#include <QGroupBox>

#include <ArmarXCore/core/logging/Logging.h>

#include <RobotAPI/libraries/aron/core/navigator/type/forward_declarations.h>
#include <RobotAPI/libraries/aron/core/Path.h>

#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>


class QGroupBox;
class QLabel;
class QSplitter;
class QTreeWidget;
class QTreeWidgetItem;


namespace armarx::armem::gui::instance
{
    class ImageView;
    class MemoryIDTreeWidgetItem;
    class WidgetsWithToolbar;


    class InstanceView : public QWidget, public armarx::Logging
    {
        Q_OBJECT
        using This = InstanceView;


    public:

        InstanceView();

        void setStatusLabel(QLabel* statusLabel);
        void setUseTypeInfo(bool enable);

        void update(const MemoryID& id, const wm::Memory& memory);
        void update(const wm::EntityInstance& instance, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        void update();

        void addInstanceView(const wm::EntityInstance& instance, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);


    signals:

        void updated();
        void instanceSelected(const MemoryID& id);
        void memoryIdResolutionRequested(const MemoryID& id);


    private slots:

        void prepareTreeContextMenu(const QPoint& pos);

        void showImageView(const aron::Path& elementPath);
        void removeImageView();


    private:

        void updateInstanceID(const MemoryID& id);
        void updateData(const aron::datanavigator::DictNavigatorPtr& data, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        void updateMetaData(const wm::EntityInstanceMetadata& metadata);
        void updateImageView(const aron::datanavigator::DictNavigatorPtr& data);

        void showErrorMessage(const std::string& message);

        aron::Path getElementPath(const QTreeWidgetItem* item) const;
        std::optional<MemoryID> getElementMemoryID(const aron::Path& elementPath);

        QAction* makeActionResolveMemoryID(const MemoryID& id);
        QAction* makeActionCopyMemoryID(const MemoryID& id);


    private:

        enum class Columns
        {
            KEY = 0,
            VALUE = 1,
            TYPE = 2,
        };

        std::optional<wm::EntityInstance> currentInstance;
        aron::typenavigator::ObjectNavigatorPtr currentAronType = nullptr;
        bool useTypeInfo = true;

        QSplitter* splitter;

        QTreeWidget* tree;
        MemoryIDTreeWidgetItem* treeItemInstanceID;
        QTreeWidgetItem* treeItemMetadata;
        QTreeWidgetItem* treeItemData;


        class ImageView : public QGroupBox
        {
        public:
            ImageView();

            instance::ImageView* view;
            aron::Path elementPath;

            WidgetsWithToolbar* toolbar;
        };
        ImageView* imageView = nullptr;

        QLabel* statusLabel = nullptr;

    };

}

namespace armarx::armem::gui
{
    using InstanceView = instance::InstanceView;
}
