#include "sanitize_typename.h"

#include <sstream>

#include <SimoxUtility/algorithm/string.h>

#include <RobotAPI/libraries/armem/aron/MemoryID.aron.generated.h>


const std::string armarx::armem::gui::instance::rawMemoryIDTypeName = armarx::armem::arondto::MemoryID::toAronType()->getName();
const std::string armarx::armem::gui::instance::sanitizedMemoryIDTypeName = "MemoryID";


namespace
{
    std::string remove_prefix(const std::string& string, const std::string& prefix)
    {
        return simox::alg::starts_with(string, prefix)
               ? string.substr(prefix.size(), std::string::npos)
               : string;
    }
    std::string remove_wrap(const std::string& string, const std::string& prefix, const std::string& suffix)
    {
        if (simox::alg::starts_with(string, prefix) && simox::alg::ends_with(string, suffix))
        {
            return string.substr(prefix.size(), string.size() - prefix.size() - suffix.size());
        }
        else
        {
            return string;
        }
    }
}

std::string armarx::armem::gui::instance::sanitizeTypeName(const std::string& typeName)
{
    if (typeName == rawMemoryIDTypeName)
    {
        return sanitizedMemoryIDTypeName;
    }

    namespace s = simox::alg;
    std::string n = typeName;
    n = s::replace_all(n, "type::Aron", "");
    n = s::replace_all(n, "data::Aron", "");

    if (false)
    {
        const std::vector<std::string> containers { "Dict", "List", "Object", "Tuple", "Pair", "NDArray" };
        for (const std::string& s : containers)
        {
            n = s::replace_all(n, s + "Type", s);
        }
    }
    else
    {
        n = s::replace_all(n, "Type", "");
    }


    n = remove_prefix(n, "Aron");
    n = remove_wrap(n, "Object<", ">");

    if (true)
    {
        const std::string del = "::";
        size_t find = n.rfind(del);
        if (find != n.npos)
        {
            find += del.size();  // include del
            std::stringstream ss;
            ss << n.substr(find) << "    (" << n.substr(0, find - del.size()) << ")";
            n = ss.str();
        }
    }

    return n;

}
