/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <unordered_map>

// Base class
#include "SerializerBase.h"

// ArmarX

namespace armarx::aron::cppserializer::detail
{
    template<typename TypenavigatorT, typename DerivedT>
    class PrimitiveSerializerBase :
        public SerializerBase<TypenavigatorT, DerivedT>
    {
    public:
        PrimitiveSerializerBase(const std::string& cppName, const std::string& aronDataTypename, const std::string& aronTypeTypename, const typename TypenavigatorT::PointerType& t) :
            SerializerBase<TypenavigatorT, DerivedT>(cppName, aronDataTypename, aronTypeTypename, t)
        {}

        virtual CppBlockPtr getResetHardBlock(const std::string& accessor) const
        {
            CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
            block_if_data->addLine(accessor + " = " + this->getFullTypenameGenerator() + "();");
            return this->ResolveMaybeResetHardBlock(accessor, block_if_data, this->typenavigator);
        }

        virtual CppBlockPtr getResetSoftBlock(const std::string& accessor) const
        {
            CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
            block_if_data->addLine(accessor + " = " + this->getFullCppTypename() + "();");
            return this->ResolveMaybeResetSoftBlock(accessor, block_if_data, this->typenavigator);
        }

        virtual CppBlockPtr getWriteBlock(const std::string& accessor) const override
        {
            CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
            std::string resolved_accessor = this->ResolveMaybeAccessor(accessor, this->typenavigator);
            block_if_data->addLine("w.writePrimitive(" + resolved_accessor + "); // of " + accessor);

            return this->ResolveMaybeWriteBlock(accessor, block_if_data, this->typenavigator);
        }

        virtual CppBlockPtr getReadBlock(const std::string& accessor) const override
        {
            CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
            std::string resolved_accessor = this->ResolveMaybeAccessor(accessor, this->typenavigator);
            block_if_data->addLine("r.readEndPrimitive(" + resolved_accessor + "); // of " + accessor);
            return this->ResolveMaybeReadBlock(accessor, "r.readStartPrimitive()", block_if_data, this->typenavigator);
        }

        virtual CppBlockPtr getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const override
        {
            CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
            block_if_data->addLine("if (not (" + accessor + " == " + otherInstanceAccessor + "))");
            block_if_data->addLineAsBlock("return false;");
            return this->ResolveMaybeEqualsBlock(accessor, otherInstanceAccessor, block_if_data, this->typenavigator);
        }
    };
}
